@@ -12,6 +12,8 @@ using System.Runtime.InteropServices;
 using Gtk;
 using Gdk;
 
+using Mono.Posix;
+
 using Beagle;
 using Beagle.Tile;
 
@@ -66,16 +68,12 @@ namespace Best {
 		Gtk.EventBox eventbox;
 		Gtk.Tooltips tips;
 		Beagle.Util.GConfXKeybinder keybinder;
-		int PosX;
-		int PosY;
 
 		[DllImport ("libtrayiconglue")]
 		private static extern IntPtr egg_tray_icon_new (string name);
 
 		public BestTray (BestWindow bw)
 		{
-			PosX = 0;
-			PosY = 0;
 			Raw = egg_tray_icon_new ("Search");
 
 			win = bw;
@@ -85,17 +83,16 @@ namespace Best {
 			eventbox.CanFocus = true;
 			eventbox.ButtonPressEvent += new ButtonPressEventHandler (ButtonPress);
 			
-			Gdk.Pixbuf smalldog = Images.GetPixbuf ("dog.png");
+			Gdk.Pixbuf smalldog = Images.GetPixbuf ("best.png");
 			eventbox.Add (new Gtk.Image (smalldog.ScaleSimple (24, 24, Gdk.InterpType.Hyper)));
 
-			string tooltip = "Bleeding-Edge Search Tool (F12)";
+			string tooltip = "Beagle Search (F12)";
 			tips = new Gtk.Tooltips ();
 			tips.SetTip (eventbox, tooltip, null);
 			tips.Enable ();
 			
-			
 			Add (eventbox);
-			ShowAll ();
+			eventbox.ShowAll ();
 
 			keybinder = new Beagle.Util.GConfXKeybinder ();
 			keybinder.Bind ("/apps/Beagle/keybindings/show_beagle",
@@ -105,13 +102,10 @@ namespace Best {
 
 		private void ShowBeaglePressed (object o, EventArgs args)
 		{
-			if (!win.Visible) {
+			if (!win.WindowIsVisible) {
 				win.Show ();
-				win.Move (PosX, PosY);
-				win.Present ();
 				win.FocusEntry ();
 			} else {
-				win.GetPosition (out PosX, out PosY);
 				win.Hide ();
 			}
 		}
@@ -120,16 +114,13 @@ namespace Best {
 		{
 			Gdk.EventButton eb = args.Event;
 			if (eb.Button == 1) {
-				if (! win.Visible) {
+				if (! win.WindowIsVisible) {
 					win.Show ();
-					win.Move (PosX, PosY);
-					win.Present ();
 					win.FocusEntry ();
 				} else {
-					win.GetPosition (out PosX, out PosY);
 					win.Hide ();
 				}
-			}else {
+			} else {
 			
 				Gtk.Menu recent_menu = MakeMenu ((Gtk.Widget) sender);
 				GuiUtils.PopupMenu (recent_menu, args.Event);
@@ -153,9 +144,6 @@ namespace Best {
 						
 			if (! win.Visible) {
 				win.Show ();
-				win.Move (PosX, PosY);
-				win.Present ();
-				
 				win.QuickSearch (quickQuery);
 			} else {
 				win.QuickSearch (quickQuery);
@@ -173,13 +161,17 @@ namespace Best {
 			// Quick Search menu items
 			ArrayList list = win.RetriveSearches ();
 			if (list == null || list.Count == 0 ) {
-				item = new Gtk.ImageMenuItem ("No Recent Searches");
+				item = new Gtk.ImageMenuItem (Catalog.GetString ("No Recent Searches"));
 				item.Sensitive = false;
 				menu.Append (item);
-			}else {
+			} else {
+				item = new Gtk.ImageMenuItem (Catalog.GetString ("Recent Searches"));
+				item.Sensitive = false;
+				item.Image = new Gtk.Image (Images.GetPixbuf ("icon-search.png"));
+				menu.Append (item);
+
 				foreach (string s in list) {
 					item = new Gtk.ImageMenuItem (s);
-					item.Image = new Gtk.Image (Images.GetPixbuf ("icon-search.png"));
 					item.Data ["Query"] = s;
 					item.Activated += new EventHandler (QuickSearchEvent);
 					menu.Append (item);
@@ -188,7 +180,7 @@ namespace Best {
 			
 			menu.Append (new Gtk.SeparatorMenuItem ());			
 		
-			item = new Gtk.ImageMenuItem ("Quit");
+			item = new Gtk.ImageMenuItem (Catalog.GetString ("Quit"));
 			item.Image = new Gtk.Image (Gtk.Stock.Quit, Gtk.IconSize.Menu);
 			item.Activated += new EventHandler (QuitEvent);
 			menu.Append (item);
