@@ -51,8 +51,11 @@ namespace Beagle.Tile {
 					Template["action:"] = "action:" + UniqueKey + "!";
 
 					if (this.Uri != null && this.Query != null) {
-						string snippet = this.Query.GetSnippet (this.Uri);
-						Template["Snippet"] = snippet;
+						SnippetRequest req = new SnippetRequest (this.Query.Text, this.Hit);
+						try {
+							SnippetResponse response = (SnippetResponse) req.Send ();
+							Template["Snippet"] = response.Snippet;
+						} catch { }
 					}
 				}
 
