@@ -56,19 +56,26 @@ namespace Beagle.Util {
 			Open           = 0x00000020, // File was opened
 			MovedFrom      = 0x00000040, // File was moved from X
 			MovedTo        = 0x00000080, // File was moved to Y
-			DeleteSubdir   = 0x00000100, // Subdir was deleted
-			DeleteFile     = 0x00000200, // Subfile was deleted
-			CreateSubdir   = 0x00000400, // Subdir was created
-			CreateFile     = 0x00000800, // Subfile was created
-			DeleteSelf     = 0x00001000, // Self was deleted
+			Create         = 0x00000100, // Subfile was created
+			Delete         = 0x00000200, // Subfile was deleted			
+			DeleteSelf     = 0x00000400, // Self was deleted
+
 			Unmount        = 0x00002000, // Backing fs was unmounted
 			QueueOverflow  = 0x00004000, // Event queue overflowed
 			Ignored        = 0x00008000, // File is no longer being watched
-			All            = 0xffffffff
+
+			IsDirectory    = 0x40000000, // Event is against a directory
+			OneShot        = 0x80000000, // Watch is one-shot
+
+			// For forward compatibility, define these explicitly
+			All            = (EventType.Access | EventType.Modify | EventType.Attrib |
+					  EventType.CloseWrite | EventType.CloseNoWrite | EventType.Open |
+					  EventType.MovedFrom | EventType.MovedTo | EventType.Create |
+					  EventType.Delete | EventType.DeleteSelf)
 		}
 
 		// Events that we want internally, even if the handlers do not
-		static private EventType base_mask =  EventType.Ignored | EventType.MovedFrom | EventType.MovedTo;
+		static private EventType base_mask =  EventType.MovedFrom | EventType.MovedTo;
 
 		/////////////////////////////////////////////////////////////////////////////////////
 
@@ -88,11 +95,6 @@ namespace Beagle.Util {
 			public uint      len;
 		}
 
-		private struct queued_event {
-			public inotify_event iev;
-			public byte[]        filename;
-		}
-
 		[DllImport ("libinotifyglue")]
 		static extern void inotify_glue_init ();
 
@@ -108,11 +110,48 @@ namespace Beagle.Util {
 								out int nr,
 								out IntPtr buffer);
 
+		/////////////////////////////////////////////////////////////////////////////////////
+
+		private class QueuedEvent {
+			public int       Wd;
+			public EventType Type;
+			public string    Filename;
+			public uint      Cookie;
+
+			public bool        Analyzed;
+			public bool        Dispatched;
+			public DateTime    HoldUntil;
+			public QueuedEvent PairedMove;
+
+			// Measured in milliseconds; 57ms is totally random
+			public const double DefaultHoldTime = 57;
+
+			public QueuedEvent ()
+			{
+				// Set a default HoldUntil time
+				HoldUntil = DateTime.Now.AddMilliseconds (DefaultHoldTime);
+			}
+
+			public void AddMilliseconds (double x)
+			{
+				HoldUntil = HoldUntil.AddMilliseconds (x);
+			}
+
+			public void PairWith (QueuedEvent other)
+			{
+				this.PairedMove = other;
+				other.PairedMove = this;
+				
+				if (this.HoldUntil < other.HoldUntil)
+					this.HoldUntil = other.HoldUntil;
+				other.HoldUntil = this.HoldUntil;
+			}
+		}
 
 		/////////////////////////////////////////////////////////////////////////////////////
 
 		static private int dev_inotify = -1;
-		static private Queue event_queue = new Queue ();
+		static private ArrayList event_queue = new ArrayList ();
 
 		static Inotify ()
 		{
@@ -123,6 +162,9 @@ namespace Beagle.Util {
 				return;
 			}
 
+			if (Environment.GetEnvironmentVariable ("BEAGLE_INOTIFY_VERBOSE") != null)
+				Inotify.Verbose = true;
+
 			inotify_glue_init ();
 
 			dev_inotify = Syscall.open ("/dev/inotify", OpenFlags.O_RDONLY);
@@ -167,9 +209,6 @@ namespace Beagle.Util {
 			}
 		}
 
-		static Hashtable cookie_hash = new Hashtable ();
-		static ArrayList cookie_list = new ArrayList ();
-
 		static public int WatchCount {
 			get { return watched_by_wd.Count; }
 		}
@@ -358,6 +397,9 @@ namespace Beagle.Util {
 
 		static unsafe void SnarfWorker ()
 		{
+			Encoding filename_encoding = Encoding.UTF8;
+			int event_size = Marshal.SizeOf (typeof (inotify_event));
+			
 			while (running) {
 
 				// We get much better performance if we wait a tiny bit
@@ -365,7 +407,6 @@ namespace Beagle.Util {
 				// FIXME: We need to be smarter here to avoid queue overflows.
 				Thread.Sleep (15);
 
-				//int N = (int) Syscall.read (dev_inotify, (void *) buffer, (IntPtr) max_snarf_size);
 				IntPtr buffer;
 				int nr;
 
@@ -381,34 +422,51 @@ namespace Beagle.Util {
 				if (nr == 0)
 					continue;
 
-				int event_size = Marshal.SizeOf (typeof (inotify_event));
+				ArrayList new_events = new ArrayList ();
+
+				bool saw_overflow = false;
+				while (nr > 0) {
+
+					// Read the low-level event struct from the buffer.
+					inotify_event raw_event;
+					raw_event = (inotify_event) Marshal.PtrToStructure (buffer, typeof (inotify_event));
+					buffer = (IntPtr) ((long) buffer + event_size);
+
+					if ((raw_event.mask & EventType.QueueOverflow) != 0)
+						saw_overflow = true;
+
+					// Now we convert our low-level event struct into a nicer object.
+					QueuedEvent qe = new QueuedEvent ();
+					qe.Wd = raw_event.wd;
+					qe.Type = raw_event.mask;
+					qe.Cookie = raw_event.cookie;
+
+					// Extract the filename payload (if any) from the buffer.
+					byte [] filename_bytes = new byte[raw_event.len];
+					Marshal.Copy (buffer, filename_bytes, 0, (int) raw_event.len);
+					buffer = (IntPtr) ((long) buffer + raw_event.len);
+					int n_chars = 0;
+					while (n_chars < filename_bytes.Length && filename_bytes [n_chars] != 0)
+						++n_chars;
+					qe.Filename = "";
+					if (n_chars > 0)
+						qe.Filename = filename_encoding.GetString (filename_bytes, 0, n_chars);
+
+					new_events.Add (qe);
+					nr -= event_size + (int) raw_event.len;
+				}
+
+				if (saw_overflow)
+					Logger.Log.Warn ("Inotify queue overflow!");
+
 				lock (event_queue) {
-					bool saw_overflow = false;
-					while (nr > 0) {
-						queued_event qe;
-
-						qe.iev = (inotify_event) Marshal.PtrToStructure (buffer, typeof (inotify_event));
-						buffer = (IntPtr) ((long) buffer + event_size);
-
-						qe.filename = new byte[qe.iev.len];
-						Marshal.Copy (buffer, qe.filename, 0, (int) qe.iev.len);
-						buffer = (IntPtr) ((long) buffer + qe.iev.len);
-
-						if (qe.iev.mask == EventType.QueueOverflow)
-							saw_overflow = true;
-						event_queue.Enqueue (qe);
-
-						nr -= event_size + (int) qe.iev.len;
-					}
-
-					if (saw_overflow)
-						Logger.Log.Warn ("Inotify queue overflow!");
-
+					event_queue.AddRange (new_events);
 					Monitor.Pulse (event_queue);
 				}
 			}
 		}
 
+
 		static public bool Verbose = false;
 
 		// Update the watched_by_path hash and the path stored inside the watch
@@ -458,10 +516,15 @@ namespace Beagle.Util {
 			if ((watched.Mask & mask) == 0)
 				return;
 
+			bool isDirectory = false;
+			if ((mask & EventType.IsDirectory) != 0)
+				isDirectory = true;
+
 			if (Verbose) {
-				Console.WriteLine ("*** inotify: {0} {1} {2} {3} {4}",
+				Console.WriteLine ("*** inotify: {0} {1} {2} {3} {4} {5}",
 						   mask, watched.Wd, watched.Path,
 						   filename != "" ? filename : "\"\"",
+						   isDirectory == true ? "(directory)" : "(file)",
 						   srcpath != null ? "(from " + srcpath + ")" : "");
 			}
 
@@ -475,98 +538,162 @@ namespace Beagle.Util {
 			}
 		}
 
-		static void DispatchWorker ()
+		////////////////////////////////////////////////////////////////////////////////////////////////////
+
+		// Dispatch-time operations on the event queue
+	       
+		static Hashtable pending_move_cookies = new Hashtable ();
+
+		// Clean up the queue, removing dispatched objects.
+		// We assume that the called holds the event_queue lock.
+		static void CleanQueue_Unlocked ()
 		{
-			Encoding filename_encoding = Encoding.UTF8;
+			int first_undispatched = 0;
+			while (first_undispatched < event_queue.Count) {
+				QueuedEvent qe = event_queue [first_undispatched] as QueuedEvent;
+				if (! qe.Dispatched)
+					break;
+				
+				if (qe.Cookie != 0)
+					pending_move_cookies.Remove (qe.Cookie);
+				
+				++first_undispatched;
+			}
 
-			while (running) {
-				queued_event qe;
+			if (first_undispatched > 0)
+				event_queue.RemoveRange (0, first_undispatched);
 
-				lock (event_queue) {
-					while (event_queue.Count == 0 && running) {
-						// Find all unmatched MovedFrom events and process them.  We expire in
-						// five seconds.  This is quite conservative, but what is the rush?
-						// Would be nice to not do this here and not have to wake up every 2 secs
-						for (int i = 0; i < cookie_list.Count; i++) {
-							PendingMove pending = cookie_list[i] as PendingMove;
-							if (pending.Time.AddSeconds (2) < DateTime.Now) {
-								SendEvent (pending.Watch, pending.SrcName, null,
-									   EventType.MovedFrom);
-								cookie_hash.Remove (pending);
-								cookie_list.Remove (pending);
-							}
+		}
+
+		// Apply high-level processing to the queue.  Pair moves,
+		// coalesce events, etc.
+		// We assume that the caller holds the event_queue lock.
+		static void AnalyzeQueue_Unlocked ()
+		{
+			int first_unanalyzed = event_queue.Count;
+			while (first_unanalyzed > 0) {
+				--first_unanalyzed;
+				QueuedEvent qe = event_queue [first_unanalyzed] as QueuedEvent;
+				if (qe.Analyzed) {
+					++first_unanalyzed;
+					break;
+				}
+			}
+			if (first_unanalyzed == event_queue.Count)
+				return;
+
+			// Walk across the unanalyzed events...
+			for (int i = first_unanalyzed; i < event_queue.Count; ++i) {
+				QueuedEvent qe = event_queue [i] as QueuedEvent;
+
+				// Pair off the MovedFrom and MovedTo events.
+				if (qe.Cookie != 0) {
+					if ((qe.Type & EventType.MovedFrom) != 0) {
+						pending_move_cookies [qe.Cookie] = qe;
+						// This increases the MovedFrom's HoldUntil time,
+						// giving us more time for the matching MovedTo to
+						// show up.
+						// (512 ms is totally arbitrary)
+						qe.AddMilliseconds (512); 
+					} else if ((qe.Type & EventType.MovedTo) != 0) {
+						QueuedEvent paired_move = pending_move_cookies [qe.Cookie] as QueuedEvent;
+						if (paired_move != null) {
+							paired_move.Dispatched = true;
+							qe.PairedMove = paired_move;
 						}
-
-						Monitor.Wait (event_queue, 2);			
 					}
-					if (!running)
-						break;
-					qe = (queued_event) event_queue.Dequeue ();
 				}
 
+				qe.Analyzed = true;
+			}
+		}
+
+		static void DispatchWorker ()
+		{
+			while (running) {
+				QueuedEvent next_event = null;
+
+				// Until we find something we want to dispatch, we will stay
+				// inside the following block of code.
+				lock (event_queue) {
+
+					while (running) {
+						CleanQueue_Unlocked ();
+
+						AnalyzeQueue_Unlocked ();
+
+						// Now look for an event to dispatch.
+						DateTime min_hold_until = DateTime.MaxValue;
+						DateTime now = DateTime.Now;
+						foreach (QueuedEvent qe in event_queue) {
+							if (qe.Dispatched)
+								continue;
+							if (qe.HoldUntil <= now) {
+								next_event = qe;
+								break;
+							}
+							if (qe.HoldUntil < min_hold_until)
+								min_hold_until = qe.HoldUntil;
+						}
+
+						// If we found an event, break out of this block
+						// and dispatch it.
+						if (next_event != null)
+							break;
+						
+						// If we didn't find an event to dispatch, we can sleep
+						// (1) until the next hold-until time
+						// (2) until the lock pulses (which means something changed, so
+						//     we need to check that we are still running, new events
+						//     are on the queue, etc.)
+						// and then we go back up and try to find something to dispatch
+						// all over again.
+						if (min_hold_until == DateTime.MaxValue)
+							Monitor.Wait (event_queue);
+						else
+							Monitor.Wait (event_queue, min_hold_until - now);
+					}
+				}
+
+				// If "running" gets set to false, we might get a null next_event as the above
+				// loop terminates
+				if (next_event == null)
+					return;
+
+				// Now we have an event, so we release the event_queue lock and do
+				// the actual dispatch.
+				
+				// Before we get any further, mark it
+				next_event.Dispatched = true;
+
 				Watched watched;
-				watched = Lookup (qe.iev.wd, qe.iev.mask);
+				watched = Lookup (next_event.Wd, next_event.Type);
 				if (watched == null)
 					continue;
 
-				// Get the filename payload, if any
-				int n_chars = 0;
-				while (n_chars < qe.filename.Length && qe.filename [n_chars] != 0)
-					++n_chars;
-				string filename = "";
-				if (n_chars > 0)
-					filename = filename_encoding.GetString (qe.filename, 0, n_chars);
-
-				// If this is a directory, we need to handle MovedTo and MovedFrom events
-				// (regardless of the actual watched.Mask) and fix up watched.Path and the
-				// path of all subdirectories.
-				//
-				// We also have to handle unmatched events.  As we presume MovedTo always
-				// follows MovedFrom, we only need to handle unmatched MoveFrom's
-				// explicitly (unmatched MovedFrom's have srcpath=null in Inotify.Event).
-				// We handle the expiration process above, in the idle loop.
-				//
-				// Unfortunately, we have to handle the Directory.Exists() test a bit
-				// roundabout, because watched.Path during MovedFrom no longer exists.
-				if (qe.iev.mask == EventType.MovedFrom) {
-					PendingMove pending = new PendingMove (watched, filename, DateTime.Now,
-									       qe.iev.cookie);
-					cookie_hash [pending.Cookie] = pending;
-					cookie_list.Add (pending);
-					continue; // Wait for a possible matching MovedTo
-				}
 				string srcpath = null;
-				if (qe.iev.mask == EventType.MovedTo) {
-					PendingMove pending = cookie_hash [qe.iev.cookie] as PendingMove;
-					if (pending != null) {
-						string dstpath = Path.Combine (watched.Path, filename);
-						srcpath = Path.Combine (pending.Watch.Path, pending.SrcName);
+
+				// If this event is a paired MoveTo, there is extra work to do.
+				if ((next_event.Type & EventType.MovedTo) != 0 && next_event.PairedMove != null) {
+					Watched paired_watched;
+					paired_watched = Lookup (next_event.PairedMove.Wd, next_event.PairedMove.Type);
+
+					if (paired_watched != null) {
+						// Set the source path accordingly.
+						srcpath = Path.Combine (paired_watched.Path, next_event.PairedMove.Filename);
+						
+						// Handle the internal rename of the directory.
+						string dstpath = Path.Combine (watched.Path, next_event.Filename);
 						if (Directory.Exists (dstpath))
 							HandleMove (srcpath, dstpath);
-						cookie_hash.Remove (pending);
-						cookie_list.Remove (pending);
-					}
-
-					// This gets prettier and prettier.  Like my Ex-Wife.  Since we delay
-					// unmatched MovedFrom events until they expire, a MovedFrom+MovedTo
-					// combination on the same file out and back into a watched directory
-					// can race and arrive as MovedTo+MovedFrom.  That is bad.
-					for (int i = 0; i < cookie_list.Count; i++) {
-						PendingMove race = cookie_list[i] as PendingMove;
-						if (race.Watch.Wd == watched.Wd && race.SrcName == filename) {
-							// Expire this MovedFrom and send it out right immediately
-							SendEvent (race.Watch, race.SrcName, null, EventType.MovedFrom);
-							cookie_hash.Remove (race);
-							cookie_list.Remove (race);
-						}
 					}
 				}
 
-				SendEvent (watched, filename, srcpath, qe.iev.mask);
+				SendEvent (watched, next_event.Filename, srcpath, next_event.Type);
 
 				// If a directory we are watching gets ignored, we need
 				// to remove it from the watchedByFoo hashes.
-				if (qe.iev.mask == EventType.Ignored) {
+				if ((next_event.Type & EventType.Ignored) != 0) {
 					lock (watched_by_wd)
 						Forget (watched);
 				}
@@ -575,6 +702,7 @@ namespace Beagle.Util {
 
 		/////////////////////////////////////////////////////////////////////////////////
 
+#if INOTIFY_TEST
 		static void Main (string [] args)
 		{
 			Queue to_watch = new Queue ();
@@ -597,9 +725,8 @@ namespace Beagle.Util {
 				Inotify.Watch (path, Inotify.EventType.All);
 
 				if (recursive) {
-					DirectoryInfo dir = new DirectoryInfo (path);
-					foreach (DirectoryInfo subdir in dir.GetDirectories ())
-						to_watch.Enqueue (subdir.FullName);
+					foreach (string subdir in DirectoryWalker.GetDirectories (path))
+						to_watch.Enqueue (subdir);
 				}
 			}
 
@@ -615,6 +742,7 @@ namespace Beagle.Util {
 			// Kill the event-reading thread so that we exit
 			Inotify.Stop ();
 		}
+#endif
 
 	}
 }
