@@ -33,20 +33,21 @@ namespace Beagle.Util {
 
 	public class Scheduler {
 
-		static private bool no_delays = false;
+		static private double global_delay = -1.0;
 		static private bool immediate_priority_only = false;
+		static private string exercise_the_dog = null;
 
 		static Scheduler ()
 		{
-			if (Environment.GetEnvironmentVariable ("EXERCISE_THE_DOG") != null) {
-				Logger.Log.Warn ("***");
-				Logger.Log.Warn ("*** The EXERCISE_THE_DOG environment variable is deprecated.");
-				Logger.Log.Warn ("*** Please use BEAGLE_EXERCISE_THE_DOG in the future.");
-				Logger.Log.Warn ("*** We will now proceed to exercise the dog.  Thank you.");
-				Logger.Log.Warn ("***");
-				no_delays = true;
-			} else if (Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG") != null) {
-				no_delays = true;
+			// We used to support the EXERCISE_THE_DOG env variable, but
+			// that was dropped.  
+			exercise_the_dog = Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG");
+			if (exercise_the_dog != null) {
+				
+				global_delay = 0.0;
+
+				if (exercise_the_dog.Length > 2 && exercise_the_dog [0] == 't')
+					global_delay = Double.Parse (exercise_the_dog.Substring (1));
 			}
 	
 			if (Environment.GetEnvironmentVariable ("BEAGLE_IMMEDIATE_PRIORITY_ONLY") != null)
@@ -561,17 +562,17 @@ namespace Beagle.Util {
 		const double default_idle_rate_factor    = 2.097; // work about 1/3rd of the time
 		const double default_maximum_delay       = 20;    // never wait for more than 20s
 
-		static DateTime first_time = DateTime.MinValue;
 		private double GetIdleTime ()
 		{
 			return SystemInformation.InputIdleTime;
 		}
 
+		// The return value and duration_of_previous_task are both measured in seconds.
 		private double ComputeDelay (Priority priority_of_next_task,
 					     double   duration_of_previous_task)
 		{
-			if (no_delays)
-				return 0;
+			if (global_delay >= 0.0)
+				return global_delay;
 
 			double rate_factor;
 
@@ -644,6 +645,7 @@ namespace Beagle.Util {
 			return new TimeSpan (ticks);
 		}
 
+#if false
 		private void DescribeTaskQueue (string note, int i0, int i1)
 		{
 			Console.WriteLine ("----------------------");
@@ -661,6 +663,7 @@ namespace Beagle.Util {
 			}
 			Console.WriteLine ("----------------------");
 		}
+#endif
 
 		// Remove nulls and cancelled tasks from the queue.
 		// Note: this does no locking!
@@ -861,6 +864,7 @@ namespace Beagle.Util {
 					collection.Clear ();
 				}
 			}
+			Logger.Log.Debug ("Scheduler.Worker finished");
 		}
 	}
 
