@@ -41,16 +41,12 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 		ArrayList summaries = new ArrayList ();
 		ArrayList mboxes = new ArrayList ();
 		
-		public MailCrawler ()
+		public MailCrawler (params string[] paths)
 		{
-			string local_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/local");
-			string imap_path = Path.Combine (PathFinder.HomeDir, ".evolution/mail/imap");
-			
-			if (Directory.Exists (local_path))
-				roots.Add (local_path);
-
-			if (Directory.Exists (imap_path))
-				roots.Add (imap_path);
+			foreach (string p in paths) {
+				if (Directory.Exists (p))
+					roots.Add (p);
+			}
 		}
 
 		private bool FileIsInteresting (FileInfo file)
@@ -72,16 +68,16 @@ namespace Beagle.Daemon.EvolutionMailDriver {
 			Queue pending = new Queue ();
 
 			foreach (string root in roots)
-				pending.Enqueue (new DirectoryInfo (root));
+				pending.Enqueue (root);
 
 			while (pending.Count > 0) {
 
-				DirectoryInfo dir = pending.Dequeue () as DirectoryInfo;
+				string dir = (string) pending.Dequeue ();
 
-				foreach (DirectoryInfo subdir in dir.GetDirectories ())
+				foreach (string subdir in DirectoryWalker.GetDirectories (dir))
 					pending.Enqueue (subdir);
 
-				foreach (FileInfo file in dir.GetFiles ("*summary")) {
+				foreach (FileInfo file in DirectoryWalker.GetFileInfos (dir)) {
 					if (file.Name == "summary") {
 						if (FileIsInteresting (file))
 							summaries.Add (file);
