@@ -41,35 +41,44 @@ namespace Beagle.Daemon {
 
 		public FileAttributes Read (string path)
 		{
-			lock (this) {
+			lock (ifas) {
 				return ifas.Read (path);
 			}
 		}
 
-		public FileAttributes ReadOrCreate (string path)
+		public FileAttributes ReadOrCreate (string path, out bool created)
 		{
-			lock (this) {
+			lock (ifas) {
+				created = false;
+
 				FileAttributes attr = ifas.Read (path);
 				if (attr == null) {
 					attr = new FileAttributes ();
-					attr.UniqueId = Guid.NewGuid ().ToString ();
+					attr.UniqueId = Guid.NewGuid ();
 					attr.Path = path;
 					ifas.Write (attr);
+					created = true;
 				}
 				return attr;
 			}
 		}
 
+		public FileAttributes ReadOrCreate (string path)
+		{
+			bool dummy;
+			return ReadOrCreate (path, out dummy);
+		}
+
 		public bool Write (FileAttributes attr)
 		{
-			lock (this) {
+			lock (ifas) {
 				return ifas.Write (attr);
 			}
 		}
 
 		public void Drop (string path)
 		{
-			lock (this) {
+			lock (ifas) {
 				ifas.Drop (path);
 			}
 		}
@@ -85,7 +94,6 @@ namespace Beagle.Daemon {
 			// FIXME: This check is incomplete, we should also check
 			// filter names, filter versions, etc.
 			return attr != null
-				&& attr.Path == path
 				&& FileSystem.GetLastWriteTime (path) <= attr.LastWriteTime;
 		}
 
@@ -93,7 +101,6 @@ namespace Beagle.Daemon {
 		{
 			FileAttributes attr = ReadOrCreate (path);
 
-			attr.Path = path;
 			attr.LastWriteTime = mtime;
 			attr.LastIndexedTime = DateTime.Now;
 
