@@ -33,7 +33,9 @@ using Beagle.Util;
 
 namespace Beagle.Daemon.FileSystemQueryable {
 
-	public class FileSystemModel {
+	public class FileSystemModel : IFileAttributesStore {
+
+		static public bool Debug = true;
 
 		public enum State {
 			Clean         = 0,
@@ -47,6 +49,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			
 			protected object big_lock;
 			protected string name;
+			protected Guid unique_id;
 			protected string parent_name; // non-null only for roots of directory trees
 			protected State state = State.Unscanned;
 			protected object watch_handle;
@@ -90,6 +93,10 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			public bool FullNameIsCached { // Sort of hacky
 				get { return cached_full_name != null; }
 			}
+
+			public Guid UniqueId {
+				get { return unique_id; }
+			}
 		
 			public int Depth {
 				get {
@@ -154,7 +161,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				}
 			}
 
-			public bool HasChildByName (string child_name)
+			public bool HasChildWithName (string child_name)
 			{
 				lock (big_lock) {
 					return children != null && children.Contains (child_name);
@@ -170,10 +177,11 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			}
 
 			// We want to always return an empty list if we have no children, not null
+			static object [] empty_list = new object [0];
 			public ICollection Children {
 				get { 
 					lock (big_lock) {
-						return children != null ? children.Values : new ArrayList ();
+						return children != null ? children.Values : empty_list;
 					}
 				}
 			}
@@ -195,14 +203,14 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				if (cmp != 0)
 					return cmp;
 
-				cmp = other.Depth - this.Depth;
-				if (cmp != 0)
-					return cmp;
-				
 				cmp = DateTime.Compare (other.last_crawl_time,
 							this.last_crawl_time);
 				if (cmp != 0)
 					return cmp;
+
+				cmp = other.Depth - this.Depth;
+				if (cmp != 0)
+					return cmp;
 				
 				return other.Name.CompareTo (this.Name);
 			}
@@ -257,12 +265,10 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				this.watch_handle = handle;
 			}
 
-			public void SetFromFileAttributes (FileAttributesStore fa_store)
+			public void SetFromFileAttributes (FileAttributes attr)
 			{
-				FileAttributes attr = fa_store.Read (FullName);
-				if (attr != null) {
-					last_crawl_time = attr.LastIndexedTime;
-				}
+				unique_id = attr.UniqueId;
+				last_crawl_time = attr.LastIndexedTime;
 			}
 
 			public void ReportActivity ()
@@ -279,6 +285,16 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			public void Rename_Unlocked (string new_name)
 			{
 				if (name != new_name) {
+					
+					if (Parent != null) {
+						DirectoryPrivate priv_parent = Parent as DirectoryPrivate;
+						priv_parent.children.Remove (name);
+						priv_parent.children [new_name] = this;
+					} else {
+						// FIXME: This is a root that is being renamed
+						// We should handle that case.
+					}
+
 					name = new_name;
 					ClearCachedName_Unlocked ();
 				}
@@ -390,23 +406,51 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		///////////////////////////////////////////////////////////////////////////
 		///////////////////////////////////////////////////////////////////////////
 		///////////////////////////////////////////////////////////////////////////
-		
+
 
 		object big_lock = new object ();
 		ArrayList roots = new ArrayList ();
 		Hashtable path_cache = new Hashtable ();
+		Hashtable by_unique_id = new Hashtable ();
 		Queue to_be_scanned = new Queue ();
 		FileNameFilter filter = new FileNameFilter ();
-		FileAttributesStore fa_store = null;
+
 		IFileEventBackend event_backend;
 		int needs_crawl_count = 0;
 		int block_activity = 0;
 
-		public FileSystemModel (FileAttributesStore fa_store,
+		UniqueIdStore unique_id_store;
+		NameIndex name_index;
+		FileAttributesStore backing_store;
+		FileAttributesStore fa_store;
+
+		public FileSystemModel (string index_directory,
+					string index_fingerprint,
 					IFileEventBackend event_backend)
 		{
-			this.fa_store = fa_store;
 			this.event_backend = event_backend;
+
+			unique_id_store = new UniqueIdStore (index_directory, index_fingerprint);
+			name_index = new NameIndex (index_directory, index_fingerprint);
+
+			IFileAttributesStore backing_store_i;
+			backing_store_i = new FileAttributesStore_Mixed (index_directory, index_fingerprint);
+			backing_store = new FileAttributesStore (backing_store_i);
+
+			// This let's us access our own implementation of the FileAttributesStore
+			// using the convenience routines in FileAttributesStore.
+			fa_store = new FileAttributesStore (this);
+		}
+
+		// I'd rather not expose these, but we really can't avoid it.
+		public UniqueIdStore UniqueIdStore {
+			get { return unique_id_store; }
+		}
+		public NameIndex NameIndex {
+			get { return name_index; }
+		}
+		public FileAttributesStore FileAttributesStore {
+			get { return fa_store; }
 		}
 
 		public ICollection Roots {
@@ -429,7 +473,11 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			DirectoryPrivate root = new DirectoryPrivate (big_lock);
 			root.InitRoot (System.IO.Path.GetDirectoryName (path), 
 				       System.IO.Path.GetFileName (path));
-			root.SetFromFileAttributes (fa_store);
+			by_unique_id [root.UniqueId] = root;
+			
+			FileAttributes attr = backing_store.ReadOrCreate (path);
+			root.SetFromFileAttributes (attr);
+			unique_id_store.AddRoot (attr.UniqueId, path, true);
 			
 			bool fire_scan_event = false;
 			lock (big_lock) {
@@ -450,14 +498,26 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				path_cache [root.FullName] = root;
 			}
 
-			if (fire_scan_event && NeedsScanEvent != null)
-				NeedsScanEvent (this);
+			if (fire_scan_event) {
+				if (NeedsScanEvent != null) {
+					NeedsScanEvent (this);
+				} else {
+					// If nothing else is listening for this event,
+					// just do it ourself.
+					ScanAll ();
+				}
+			}
 
 			return root;
 		}
 
 		///////////////////////////////////////////////////////////////////////////
 
+		public Directory GetDirectoryByUniqueId (Guid uid)
+		{
+			return by_unique_id [uid] as Directory;
+		}
+
 		public Directory GetDirectoryByPath (string path)
 		{
 			// FIXME: We should probably canonize the path and
@@ -523,14 +583,132 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 		///////////////////////////////////////////////////////////////////////////
 
-		public bool Ignore (string path)
+		// This works around a mono bug: the DateTimes that we get out of stat
+		// don't correctly account for daylight savings time.  We declare the two
+		// dates to be equal if:
+		// (1) They actually are equal
+		// (2) The first date is exactly one hour ahead of the second
+		static private bool DatesAreTheSame (DateTime system_io_datetime, DateTime stat_datetime)
 		{
-			return filter.Ignore (path) || FileSystem.IsSymLink (path);
+			double t = (system_io_datetime - stat_datetime).TotalSeconds;
+			return Math.Abs (t) < 1e-5 || Math.Abs (t-3600) < 1e-5;
 		}
 
-		public bool IsUpToDate (string path)
+		public enum RequiredAction {
+			None,
+			Index,
+			Rename
+		}
+		
+		// When we return RequiredAction.Rename, previous_path is set to the last
+		// known name.  Otherwise it is null.
+		public RequiredAction DetermineRequiredAction (string path, out string previous_path)
 		{
-			return this.fa_store.IsUpToDate (path);
+			if (Debug)
+				Logger.Log.Debug ("*** What should we do with {0}?", path);
+			previous_path = null;
+
+			if (Ignore (path)) {
+				if (Debug)
+					Logger.Log.Debug ("*** Ignoring {0}", path);
+				return RequiredAction.None;
+			}
+
+			FileAttributes attr = this.backing_store.Read (path);
+			if (attr == null) {
+				if (Debug)
+					Logger.Log.Debug ("*** No attributes on {0}", path);
+				return RequiredAction.Index;
+			}
+
+			Mono.Posix.Stat stat;
+			try {
+				Mono.Posix.Syscall.stat (path, out stat);
+			} catch (Exception ex) {
+				Logger.Log.Debug ("Caught exception stating {0}", path);
+				Logger.Log.Debug (ex);
+				return RequiredAction.None;
+			}
+
+			// If the file has changed since we put on the
+			// attributes, index.
+			if (! DatesAreTheSame (attr.LastWriteTime, stat.MTime)) {
+				if (Debug)
+					Logger.Log.Debug ("*** mtime has changed on {0}", path);
+				
+				// If the mtime has changed and the path doesn't match the
+				// unique ID, the file was probably copied.  Drop the file attributes
+				// so that a new unique ID will be assigned when we index.
+				if (PathFromUid (attr.UniqueId) != path)
+					this.backing_store.Drop (path);
+
+				return RequiredAction.Index;
+			}
+
+			// If the inode data has changed since it was last
+			// indexed, we might have been moved or copied.
+			if (! DatesAreTheSame (attr.LastIndexedTime, stat.CTime)) {
+				string path_from_uid = PathFromUid (attr.UniqueId);
+				if (Debug)
+					Logger.Log.Debug ("CTime check {0} {1} '{2}'", attr.LastIndexedTime, stat.CTime, path_from_uid);
+				if (path_from_uid == null) {
+					if (Debug)
+						Logger.Log.Debug ("*** Unfamiliar Uid, indexing {0}", path);
+					return RequiredAction.Index;
+				} else if (path_from_uid != path) {
+					
+					// Check to see the file mentioned in path_from_uid still exists.
+					// If so, path_from_uid is probably a copy of path and should be
+					// indexed separately.
+					// FIXME: This probably shouldn't happen --- if it is a copy, then the
+					// mtime must have changed, right?  Maybe path_from_uid is actually the
+					// copy, and we should strip it's attrs and treat this as a rename.
+
+					bool looks_like_copy = false;
+					try {
+						FileAttributes other_attr = this.Read (path_from_uid);
+						if (other_attr != null && other_attr.UniqueId == attr.UniqueId) {
+							Logger.Log.Debug ("*** '{0}' looks like a copy of '{1}'", path, path_from_uid);
+							looks_like_copy = true;
+						}
+					} catch (Exception ex) {
+						Logger.Log.Debug ("*** Caught exception while inspecting '{0}'", path_from_uid);
+						Logger.Log.Debug (ex);
+					}
+
+					// If we think this is a copy, clear out the attributes and treat it like
+					// a new file.
+					if (looks_like_copy) {
+						if (Debug)
+							Logger.Log.Debug ("*** Stripping attributes from {0} and requesting new indexing", path);
+						this.backing_store.Drop (path);
+						return RequiredAction.Index;
+					}
+					
+					if (Debug)
+						Logger.Log.Debug ("*** Path has changed, renaming {0} => {1}", previous_path, path);
+					previous_path = path_from_uid;
+					return RequiredAction.Rename;
+				}
+			}
+
+			if (Debug)
+				Logger.Log.Debug ("*** Doing nothing to {0}", path);
+
+			return RequiredAction.None;
+		}
+
+		// We use this to test whether or not we need to work around
+		// http://bugzilla.ximian.com/show_bug.cgi?id=71214
+		private static bool MonoWillBreakThisPath (string path)
+		{
+			Uri uri = UriFu.PathToFileUri (path);
+			return uri.LocalPath != path; // This assumes that the path is absolute.
+		}
+		
+		public bool Ignore (string path)
+		{
+			return filter.Ignore (path) || FileSystem.IsSymLink (path) || MonoWillBreakThisPath (path);
 		}
 
 		///////////////////////////////////////////////////////////////////////////
@@ -546,7 +724,7 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		{
 			DirectoryPrivate priv = (DirectoryPrivate) dir;
 
-			if (dir.State == State.Unscanned)
+			if (dir.State == State.Unscanned && event_backend != null)
 				priv.SetWatchHandle (event_backend.WatchDirectories (priv.FullName));
 
 			Hashtable known_children = null;
@@ -557,13 +735,20 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			}
 
 			System.IO.DirectoryInfo info = new System.IO.DirectoryInfo (priv.FullName);
-			foreach (System.IO.DirectoryInfo subinfo in info.GetDirectories ()) {
-				if (! Ignore (subinfo.FullName)) {
-					if (! priv.HasChildByName (subinfo.Name))
-						AddChild_Unlocked (priv, subinfo.Name);
+
+			// It's the call to GetDirectoryInfos() that may
+			// trigger the exception caught below.
+			try {
+				foreach (System.IO.DirectoryInfo subinfo in DirectoryWalker.GetDirectoryInfos (info)) {
+					if (! Ignore (subinfo.FullName)) {
+						if (! priv.HasChildWithName (subinfo.Name))
+							AddChild_Unlocked (priv, subinfo.Name);
+					}
+					if (known_children != null)
+						known_children.Remove (subinfo.Name);
 				}
-				if (known_children != null)
-					known_children.Remove (subinfo.Name);
+			} catch (System.IO.DirectoryNotFoundException e) {
+				Logger.Log.Warn ("Skipping over {0}: {1}", priv.FullName, e.Message);
 			}
 
 			if (known_children != null) {
@@ -600,10 +785,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			bool fire_crawl_event = false;
 			lock (big_lock) {
 				int old_needs_crawl_count = needs_crawl_count;
-				while (true) {
+				while (to_be_scanned.Count > 0) {
 					Directory dir;
-					if (to_be_scanned.Count == 0)
-						break;
 					dir = to_be_scanned.Dequeue () as Directory;
 					ScanOne_Unlocked (dir);
 					need_watches.Add (dir);
@@ -612,7 +795,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
 
 				foreach (DirectoryPrivate priv in need_watches) {
 					if (priv.NeedsFinalWatches) {
-						priv.SetWatchHandle (event_backend.WatchFiles (priv.FullName, priv.WatchHandle));
+						if (event_backend != null)
+							priv.SetWatchHandle (event_backend.WatchFiles (priv.FullName, priv.WatchHandle));
 						priv.NeedsFinalWatches = false;
 					}
 				}
@@ -691,7 +875,8 @@ namespace Beagle.Daemon.FileSystemQueryable {
 					// Re-scan post-crawl
 					ScanOne_Unlocked (priv);
 					if (priv.NeedsFinalWatches) {
-						priv.SetWatchHandle (event_backend.WatchFiles (priv.FullName, priv.WatchHandle));
+						if (event_backend != null)
+							priv.SetWatchHandle (event_backend.WatchFiles (priv.FullName, priv.WatchHandle));
 						priv.NeedsFinalWatches = false;
 					}
 
@@ -699,13 +884,13 @@ namespace Beagle.Daemon.FileSystemQueryable {
 					priv.SetState (State.PossiblyClean);
 				}					
 
-				FileAttributes attr = fa_store.ReadOrCreate (priv.FullName);
+				FileAttributes attr = backing_store.ReadOrCreate (priv.FullName);
 				attr.LastIndexedTime = priv.LastCrawlTime;
 
 				// FIXME: We should check the return value and make sure that
 				// the write succeeds.  (But what is the right behavior if it
 				// fails?)
-				fa_store.Write (attr);
+				backing_store.Write (attr);
 				
 			}
 		}
@@ -718,6 +903,17 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			}
 		}
 
+		// This is used by the FSW backend when a subdirectory
+		// below a watched directory reports a change.
+		public void ReportChanges (Directory dir)
+		{
+			lock (big_lock) {
+				DirectoryPrivate priv = (DirectoryPrivate) dir;
+				priv.SetState (State.Dirty);
+				priv.ReportActivity ();
+			}
+		}
+
 		public void SetAllToUnknown ()
 		{
 			lock (big_lock) {
@@ -733,7 +929,12 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			DirectoryPrivate child = new DirectoryPrivate (big_lock);
 			child.Rename_Unlocked (child_name);
 			parent.AddChild_Unlocked (child);
-			child.SetFromFileAttributes (fa_store);
+			by_unique_id [child.UniqueId] = child;
+
+			FileAttributes attr = backing_store.ReadOrCreate (child.FullName);
+			child.SetFromFileAttributes (attr);
+			unique_id_store.Add (child.UniqueId, parent.UniqueId, child.Name, true);
+
 			to_be_scanned.Enqueue (child);
 					
 		}
@@ -756,6 +957,9 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			DirectoryPrivate priv = (DirectoryPrivate) dir;
 			
 			lock (big_lock) {
+				by_unique_id.Remove (priv.UniqueId);
+				unique_id_store.Drop (priv.UniqueId);
+
 				RecursivelyRemoveFromPathCache_Unlocked (priv);
 				priv.Detatch_Unlocked ();
 			}
@@ -768,6 +972,12 @@ namespace Beagle.Daemon.FileSystemQueryable {
 			lock (big_lock) {
 				RecursivelyRemoveFromPathCache_Unlocked (priv);
 				priv.Rename_Unlocked (new_name);
+
+				// Write the new name out to the store
+				unique_id_store.Add (priv.UniqueId, 
+						     priv.Parent != null ? priv.Parent.UniqueId : Guid.Empty,
+						     new_name,
+						     true);
 			}
 		}
 
@@ -780,6 +990,9 @@ namespace Beagle.Daemon.FileSystemQueryable {
 				RecursivelyRemoveFromPathCache_Unlocked (priv);
 				priv.Detatch_Unlocked ();
 				new_parent_priv.AddChild_Unlocked (priv);
+
+				// Write the new parent dir out to the store
+				unique_id_store.Add (priv.UniqueId, new_parent.UniqueId, priv.Name, true);
 			}
 		}
 
@@ -805,6 +1018,114 @@ namespace Beagle.Daemon.FileSystemQueryable {
 		}
 
 		//////////////////////////////////////////////////////////////////////////////////
+
+		// Search by filename
+
+		// Returns a collection of internal Uris
+		public ICollection Search (Query query, ICollection list_of_uris)
+		{
+			return name_index.Search (query, list_of_uris);
+		}
+
+		//////////////////////////////////////////////////////////////////////////////////
+
+		public string PathFromUid (Guid uid)
+		{
+			return unique_id_store.GetPathById (uid);
+		}
+
+		public Guid PathToUid (string path)
+		{
+			Guid unique_id;
+
+			// FIXME: This is probably racey.
+			if (System.IO.File.Exists (path) || System.IO.Directory.Exists (path)) {
+				FileAttributes attr;
+				attr = backing_store.ReadOrCreate (path);
+				unique_id = attr.UniqueId;
+			} else {
+				// Maybe the file got deleted.  If so, try to get it from
+				// the unique id store.
+				string dir_name = System.IO.Path.GetDirectoryName (path);
+				string file_name = System.IO.Path.GetFileName (path);
+				Directory dir = GetDirectoryByPath (dir_name);
+				unique_id = unique_id_store.GetIdByNameAndParentId (file_name, dir.UniqueId);
+			}
+
+			return unique_id;
+		}
+		
+		// Map between internal and external Uris.
+
+		public Uri PathToInternalUri (string path)
+		{
+			return GuidFu.ToUri (PathToUid (path));
+		}
+
+		public Uri ToInternalUri (Uri external_uri)
+		{
+			return PathToInternalUri (external_uri.LocalPath);
+		}
+
+		public Uri FromInternalUri (Uri internal_uri)
+		{
+			return unique_id_store.GetFileUriByUidUri (internal_uri);
+		}
+
+		public bool InternalUriIsValid (Uri internal_uri)
+		{
+			string path = unique_id_store.GetPathByUidUri (internal_uri);
+			if (path == null)
+				return false;
+			return System.IO.File.Exists (path) || System.IO.Directory.Exists (path);
+		}
+
+		//////////////////////////////////////////////////////////////////////////////////
+
+		//
+		// Implementation of IFileAttributesStore
+		//
+
+		public FileAttributes Read (string path)
+		{
+			FileAttributes attr;
+			attr = backing_store.Read (path);
+			return attr;
+		}
+
+		public bool Write (FileAttributes attr)
+		{
+			bool write_rv = backing_store.Write (attr);
+
+			// Since directories are always explicitly cached,
+			// this check is equivalent to checking that
+			// attr.Path is a file and not a directory.
+			if (! unique_id_store.IsCached (attr.UniqueId)) {
+				// When writing out our FileAttributes, add the
+				// file and unique id to the UniqueIdStore.
+				string dir_name = System.IO.Path.GetDirectoryName (attr.Path);
+				string file_name = System.IO.Path.GetFileName (attr.Path);
+				Directory dir = GetDirectoryByPath (dir_name);
+				unique_id_store.Add (attr.UniqueId, dir.UniqueId, file_name, false);
+			}
+
+			return write_rv;
+		}
+
+		public void Drop (string path)
+		{
+			Guid unique_id = PathToUid (path);
+			unique_id_store.Drop (unique_id);
+
+			backing_store.Drop (path);
+		}
+
+		//////////////////////////////////////////////////////////////////////////////////
+
+		public void DropUid (Guid unique_id)
+		{
+			unique_id_store.Drop (unique_id);
+		}
 		
 	}
 }
