@@ -36,35 +36,34 @@ using Beagle.Util;
 
 namespace Beagle.Daemon.LauncherQueryable {
 
-	[QueryableFlavor (Name="Launcher", Domain=QueryDomain.Local)]
+	[QueryableFlavor (Name="Launcher", Domain=QueryDomain.Local, RequireInotify=false)]
 	public class LauncherQueryable : LuceneQueryable {
 
 		private static Logger log = Logger.Get ("LauncherQueryable");
 		ArrayList Dirs;
 		Hashtable watched = new Hashtable ();
-		string home;
-		FileStream LauncherDB;
+		int polling_interval_in_hours = 1;
 
 		public LauncherQueryable () : base ("LauncherIndex")
 		{
 			Dirs = new ArrayList ();
 			// Add GNOME dirs
-			string prefix = ExternalStringsHack.GnomePrefix;
-			StringBuilder fullpath = new StringBuilder (prefix);
-			fullpath.Replace ("\n","");
-			fullpath.Append ("/share/");
-			Dirs.Add (fullpath.ToString () + "applications/");
-			Dirs.Add (fullpath.ToString () + "gnome/apps/");
-			Dirs.Add (fullpath.ToString () + "control-center-2.0/");
-			Dirs.Add (fullpath.ToString () + "control-center/");
-			home = Environment.GetEnvironmentVariable ("HOME");
-			Dirs.Add (home + ".gnome2/panel2.d/default/launchers/");
+			string path = Path.Combine (ExternalStringsHack.GnomePrefix, "share");
+			Dirs.Add (Path.Combine (path, "applications"));
+			Dirs.Add (Path.Combine (path, "gnome/apps"));
+			Dirs.Add (Path.Combine (path, "control-center-2.0"));
+			Dirs.Add (Path.Combine (path, "control-center"));
+			Dirs.Add (Path.Combine (PathFinder.HomeDir, ".gnome2/panel2.d/default/launchers"));
 
-			// FIXME:  Add KDE dirs
-			// home = Environment.GetEnvironmentVariable ("KDEDIR");
-			// Dirs.Add (home + "share/services/");
-			// Dirs.Add (home + "share/apps/");
-			// Dirs.Add (home + "share/applications/");
+			// Add KDE dirs
+			foreach (string kde_dir in KdeUtils.KdeLocations) {
+				if (kde_dir == null || kde_dir == String.Empty)
+					continue;
+
+				string share_dir = Path.Combine (kde_dir, "share");
+				Dirs.Add (Path.Combine(share_dir, "applications"));
+			}
+			Dirs.Add (Path.Combine (PathFinder.HomeDir, ".local/share/applications"));
 		}
 
 		override protected IFileAttributesStore BuildFileAttributesStore (string index_fingerprint) 
@@ -75,18 +74,48 @@ namespace Beagle.Daemon.LauncherQueryable {
 		public override void Start ()
 		{
 			base.Start ();
-			Inotify.Event += OnInotifyEvent;
-			log.Info ("Scanning Launchers");
+
+			log.Info ("Starting launcher backend");
 			Stopwatch timer = new Stopwatch ();
 			timer.Start ();
-			int LaunchersFound = 0;
-			foreach (String dir in Dirs)
-				LaunchersFound += Watch (dir);
+
+			if (Inotify.Enabled)
+				Inotify.Event += OnInotifyEvent;
+
+			Crawl ();
+
+			if (!Inotify.Enabled) {
+				Scheduler.Task task = Scheduler.TaskFromHook (new Scheduler.TaskHook (CrawlHook));
+				task.Tag = "Crawling system launchers";
+				ThisScheduler.Add (task);
+			}
+
 			timer.Stop ();
-			log.Info ("Found {0} Launchers in {1}",LaunchersFound, timer);
+			log.Info ("Launcher backend worker thread done in {0}", timer);
 		}
 
-		private int Watch (string path)
+		private void Crawl ()
+		{
+			log.Info ("Scanning Launchers");
+			Stopwatch timer = new Stopwatch ();
+			int launchers_found = 0;
+			foreach (String dir in Dirs)
+				launchers_found += CrawlLaunchers (dir);
+
+			log.Info ("Found {0} Launchers in {1}", launchers_found, timer);
+		}
+
+		private void CrawlHook (Scheduler.Task task)
+		{
+			Crawl ();
+			task.Reschedule = true;
+			task.TriggerTime = DateTime.Now.AddHours (this.polling_interval_in_hours);
+		}
+
+		// Crawl the specified directory and all subdirectories, indexing all
+		// discovered launchers. If Inotify is available, every directory 
+		// scanned will be added to the watched list.
+		private int CrawlLaunchers (string path)
 		{
 			DirectoryInfo root = new DirectoryInfo (path);
 			if (! root.Exists)
@@ -98,8 +127,12 @@ namespace Beagle.Daemon.LauncherQueryable {
 
 			while (queue.Count > 0) {
 				DirectoryInfo dir = queue.Dequeue () as DirectoryInfo;
-				int wd = Inotify.Watch (dir.FullName, Inotify.EventType.CreateSubdir | Inotify.EventType.Modify);
-				watched [wd] = true;
+				
+				if (Inotify.Enabled) {
+					int wd = Inotify.Watch (dir.FullName, Inotify.EventType.Create | Inotify.EventType.Modify);
+					watched [wd] = true;
+				}
+
 				foreach (FileInfo file in dir.GetFiles ()) {
 					IndexLauncher (file, Scheduler.Priority.Delayed);
 					++fileCount;
@@ -115,21 +148,21 @@ namespace Beagle.Daemon.LauncherQueryable {
 		private void OnInotifyEvent (int wd,
 				  	    string path,
 					    string subitem,
-					    Inotify.EventType type,
-					    uint cookie)
+					    string srcpath,
+					    Inotify.EventType type)
 		{
 			if (subitem == "" || ! watched.Contains (wd))
 				return;
 
 			string fullPath = Path.Combine (path, subitem);
 
-			switch (type) {
-				case Inotify.EventType.CreateSubdir:
-					Watch (fullPath);
-					break;
-				case Inotify.EventType.Modify:
-					IndexLauncher (new FileInfo (fullPath), Scheduler.Priority.Immediate);
-					break;
+			if ((type & Inotify.EventType.Create) != 0 && (type & Inotify.EventType.IsDirectory) != 0) {
+				CrawlLaunchers (fullPath);
+				return;
+			}
+			if ((type & Inotify.EventType.Modify) != 0) {
+				IndexLauncher (new FileInfo (fullPath), Scheduler.Priority.Immediate);
+				return;
 			}
 		}
 
@@ -140,39 +173,58 @@ namespace Beagle.Daemon.LauncherQueryable {
 				return;
 			
 			/* Check to see if file is a launcher */
-			if (Beagle.Util.VFS.Mime.GetMimeType (file.ToString ()) != "application/x-desktop")
+			if (Beagle.Util.VFS.Mime.GetMimeType (file.FullName) != "application/x-desktop")
 				return;
-			StreamReader reader = new StreamReader (file.Open (FileMode.Open, FileAccess.Read, FileShare.Read));
+
+			StreamReader reader;
+
+			try {
+				reader = new StreamReader (file.Open (FileMode.Open, FileAccess.Read, FileShare.Read));
+			} catch (Exception e) {
+				log.Warn ("Could not open '{0}': {1}", file.FullName, e.Message);
+				return;
+			}
+
 			if (reader.ReadLine () != "[Desktop Entry]") {
 				reader.Close ();
 				return;
 			}
-				
+
 			/* I'm convinced it is a launcher */
 			Indexable indexable = new Indexable (UriFu.PathToFileUri (file.FullName));
 
 			indexable.Timestamp = file.LastWriteTime;
 			indexable.Type = "Launcher";
 			indexable.MimeType = "application/x-desktop";
+			
+			// desktop files must have a name
+			bool have_name = false;
 
 			String line;
 			while ((line = reader.ReadLine ()) != null)  {
 				string [] sline = line.Split ('=');
 				if (sline.Length != 2)
 					continue;
-				if (!sline[1].Equals (""))  {
-					// FIXME:  add other language support
-					if ((sline[0].Equals ("Exec")) || (sline[0].Equals ("Icon")) || (sline[0].Equals ("Name")) || (sline[0].Equals ("Comment"))) {
-						StringBuilder property = new StringBuilder ("fixme:");
-						indexable.AddProperty (Beagle.Property.NewKeyword (property.Append(sline[0]).ToString (), sline[1]));
-					}
+
+				// FIXME: We shouldnt really search fields that are in other locales than the current should we?
+
+				if (sline [0].Equals ("Icon") || sline [0].Equals ("Exec")) {
+					indexable.AddProperty (Beagle.Property.NewUnsearched ("fixme:" + sline[0], sline[1]));
+				} else if (sline [0].StartsWith ("Name")) {
+					if (sline [0] == "Name")
+						have_name = true;
+					indexable.AddProperty (Beagle.Property.NewKeyword ("fixme:" + sline[0], sline[1]));
+				} else if (sline[0].StartsWith ("Comment")) {
+					   indexable.AddProperty (Beagle.Property.NewKeyword ("fixme:" + sline[0], sline[1]));
 				}
 			}
 			reader.Close ();
-			Scheduler.Task task = NewAddTask (indexable);
-			task.Priority = priority;
-			ThisScheduler.Add (task);
+			
+			if (have_name) {
+				    Scheduler.Task task = NewAddTask (indexable);
+				    task.Priority = priority;
+				    ThisScheduler.Add (task);
+			}
 		}
 	}
 }
-
