@@ -53,6 +53,7 @@ namespace Beagle.Daemon {
 	public class LuceneDriver : IIndexer {
 
 		public delegate bool UriFilter (Uri uri);
+		public delegate Uri UriRemapper (Uri uri);
 		public delegate double RelevancyMultiplier (Hit hit);
 
 		public event IIndexerChangedHandler ChangedEvent;
@@ -69,7 +70,13 @@ namespace Beagle.Daemon {
 		// 7: incremented to force a re-index after our upgrade to lucene 1.4
 		//    (in theory the file formats are compatible, we are seeing 'term
 		//    out of order' exceptions in some cases)
-		private const int VERSION = 7;
+		// 8: another forced re-index, this time because of massive changes
+		//    in the file system backend (it would be nice to have per-backend
+		//    versioning so that we didn't have to purge all indexes just
+		//    because one changed)
+		// 9: changed the way properties are stored, changed in conjunction
+		//    with sane handling of multiple properties on hits.
+		private const int MAIN_VERSION = 9;
 
 		private string top_dir;
 		private Hashtable pending_by_uri = UriFu.NewHashtable ();
@@ -79,9 +86,11 @@ namespace Beagle.Daemon {
 		private bool optimizing = false;
 		private int last_item_count = -1;
 
-		public LuceneDriver (string index_name)
+		public LuceneDriver (string index_name) : this (index_name, -1) { }
+
+		public LuceneDriver (string index_name, int index_version)
 		{
-			Setup (index_name);
+			Setup (index_name, index_version);
 		}
 
 		public string IndexDirectory {
@@ -108,16 +117,14 @@ namespace Beagle.Daemon {
 
 		/////////////////////////////////////////////////////
 
-
-		private void Setup (string index_name)
-		{
-			top_dir = Path.Combine (PathFinder.RootDir, index_name); 
+		private void Setup (string index_name, int index_version)
+		{			
+			top_dir = Path.Combine (PathFinder.StorageDir, index_name); 
 			
 			string versionFile = Path.Combine (top_dir, "version");
 			string fingerprintFile = Path.Combine (top_dir, "fingerprint");
 			string lockDir = Path.Combine (top_dir, "Locks");
 			string indexDir = Path.Combine (top_dir, "Index");
-			string queueDir = Path.Combine (top_dir, "Queue");
 			string indexTestFile = Path.Combine (indexDir, "segments");
 
 			bool versionExists = File.Exists (versionFile);
@@ -130,8 +137,18 @@ namespace Beagle.Daemon {
 				StreamReader sr = new StreamReader (versionFile);
 				string versionStr = sr.ReadLine ();
 				sr.Close ();
-				
-				if (versionStr != Convert.ToString (VERSION))
+
+				int old_main_version, old_index_version = -1;
+				int i = versionStr.IndexOf (".");
+
+				if (i != -1) {
+					old_main_version = Convert.ToInt32 (versionStr.Substring (0,i));
+					old_index_version = Convert.ToInt32 (versionStr.Substring (i+1));
+				} else {
+					old_main_version = Convert.ToInt32 (versionStr);
+				}
+
+				if (old_main_version != MAIN_VERSION || old_index_version != index_version)
 					indexExists = false;
 			}
 
@@ -188,13 +205,12 @@ namespace Beagle.Daemon {
 
 				// Write out our version information
 				sw = new StreamWriter (versionFile, false);
-				sw.WriteLine ("{0}", VERSION);
+				sw.WriteLine ((index_version == -1) ? "{0}" : "{0}.{1}", MAIN_VERSION, index_version);
 				sw.Close ();
 			}
 
 			Lucene.Net.Store.FSDirectory store;
-			store = Lucene.Net.Store.FSDirectory.GetDirectory (indexDir, false);
-			store.TempDirectoryName = lockDir;
+			store = Lucene.Net.Store.FSDirectory.GetDirectory (indexDir, lockDir, false);
 			ourStore = store;
 			ourStorePath = indexDir;
 
@@ -237,6 +253,8 @@ namespace Beagle.Daemon {
 		// Public Indexing API
 		//
 
+		static object [] empty_collection = new object [0];
+
 		public void Add (Indexable indexable)
 		{
 			Uri uri = indexable.Uri;
@@ -259,6 +277,13 @@ namespace Beagle.Daemon {
 			}
 		}
 
+		public void Rename (Uri old_uri, Uri new_uri)
+		{
+			Logger.Log.Error ("**** LuceneDriver does not support Rename!");
+			Logger.Log.Error ("**** old_uri={0}", old_uri);
+			Logger.Log.Error ("**** new_uri={0}", new_uri);
+		}
+
 		public int PendingAdds {
 			get { return pending_adds; }
 		}
@@ -326,8 +351,8 @@ namespace Beagle.Daemon {
 			last_item_count = reader.NumDocs ();
 			reader.Close ();
 			watch.Stop ();
-			//Log.Debug ("Step #1: {0} {1} {2}", watch, pending_uris.Count,
-			//	   watch.ElapsedTime / pending_uris.Count);
+			Log.Debug ("Lucene Delete: {0} {1} {2}", watch, pending_uris.Count,
+				   pending_uris.Count / watch.ElapsedTime);
 
 
 			// Step #2: Write out the pending adds
@@ -335,7 +360,14 @@ namespace Beagle.Daemon {
 			IndexWriter writer = null;
 			foreach (Indexable indexable in pending_indexables) {
 				
-				Log.Debug ("+ {0}", indexable.Uri);
+				Log.Debug ("+ {0}", indexable.DisplayUri);
+
+				try {
+					FilterFactory.FilterIndexable (indexable);
+				} catch (Exception e) {
+					Log.Error ("Unable to filter {0} (mimetype={1})", indexable.DisplayUri, indexable.MimeType);
+					Log.Error (e);
+				}
 				
 				Document doc = null;
 				try {
@@ -357,12 +389,13 @@ namespace Beagle.Daemon {
 			if (writer != null) 
 				writer.Close ();
 			watch.Stop ();
-			//Log.Debug ("Step #2: {0}", watch);
+			Log.Debug ("Lucene Add: {0} {1} {2}", watch, pending_indexables.Count,
+				   pending_indexables.Count / watch.ElapsedTime);
 			
 
 			// Step #3: Fire off an event telling what we just did.
 			if (ChangedEvent != null) {
-				ChangedEvent (this, added_uris, removed_uris);
+				ChangedEvent (this, added_uris, removed_uris, empty_collection);
 			}
 
 			lock (pending_by_uri)
@@ -396,32 +429,9 @@ namespace Beagle.Daemon {
 			Stopwatch watch = new Stopwatch ();
 			watch.Start ();
 
-			bool optimize_in_process = true;
-
-#if false
-			if (optimizer_path != null) {
-				try {
-					// Try to optimize in an external process
-					Process p = new Process ();
-					p.StartInfo.UseShellExecute = true;
-					p.StartInfo.FileName = optimizer_path;
-					p.StartInfo.Arguments = StorePath;
-					p.Start ();
-					p.WaitForExit ();
-					optimize_in_process = false;
-				} catch (Exception ex) {
-					Log.Debug ("Caught exception while running beagle-index-optimizer");
-					Log.Debug (ex);
-				}
-			}
-#endif
-
-			if (optimize_in_process) {
-				Log.Debug ("Optimizing in-process.");
-				IndexWriter writer = new IndexWriter (Store, null, false);
-				writer.Optimize ();
-				writer.Close ();
-			}
+			IndexWriter writer = new IndexWriter (Store, null, false);
+			writer.Optimize ();
+			writer.Close ();
 
 			watch.Stop ();
 
@@ -437,10 +447,12 @@ namespace Beagle.Daemon {
 
 		// Returns the lowest matching score before the results are
 		// truncated.
-		public void DoQuery (QueryBody           body,
+		public void DoQuery (Query               query,
 				     IQueryResult        result,
-				     ICollection         list_of_uris,
+				     ICollection         search_subset, // should be internal uris
+				     ICollection         bonus_uris,    // should be internal uris
 				     UriFilter           uri_filter,
+				     UriRemapper         uri_remapper, // map to external uris
 				     RelevancyMultiplier relevancy_multiplier)
 		{
 			double t_lucene;
@@ -449,13 +461,11 @@ namespace Beagle.Daemon {
 			Stopwatch sw = new Stopwatch ();
 			sw.Start ();
 			IndexReader reader = IndexReader.Open (Store);
-			if (last_item_count == -1)
-				last_item_count = reader.NumDocs ();
 
 			LNS.Searcher searcher = new LNS.IndexSearcher (reader);
-			LNS.Query query = ToLuceneQuery (body, list_of_uris);
+			LNS.Query lucene_query = ToLuceneQuery (query, search_subset, bonus_uris);
 
-			LNS.Hits hits = searcher.Search (query);
+			LNS.Hits hits = searcher.Search (lucene_query);
 			sw.Stop ();
 
 			t_lucene = sw.ElapsedTime;
@@ -469,7 +479,6 @@ namespace Beagle.Daemon {
 			if (n_hits == 0)
 				return;
 
-			ArrayList top_hits = new ArrayList ();
 			for (int i = 0; i < n_hits; ++i) {
 				Document doc = hits.Doc (i);
 
@@ -487,12 +496,13 @@ namespace Beagle.Daemon {
 				}
 
 				Hit hit = FromLuceneDocToHit (doc, hits.Id (i), score);
+				if (uri_remapper != null)
+					hit.Uri = uri_remapper (hit.Uri);
 
 				if (relevancy_multiplier != null) {
 					double m = relevancy_multiplier (hit);
 					hit.ScoreMultiplier = (float) m;
 				}
-
 				result.Add (hit);
 			}
 
@@ -505,10 +515,11 @@ namespace Beagle.Daemon {
 			// The call to searcher.Close () also closes the IndexReader.
 			searcher.Close ();
 
-			log.Debug ("{0}: n_hits={1} lucene={2:0.00}s  assembly={3:0.00}s",
+			log.Debug ("{0}: n_hits={1} lucene={2:0.00}s assembly={3:0.00}s",
 				   StorePath, n_hits, t_lucene, t_assembly);
 		}
-
+		
+		// FIXME: This should support Uri filtering, Uri remapping, etc.
 		public ICollection DoQueryByUri (ICollection list_of_uris)
 		{
 			LNS.BooleanQuery uri_query = new LNS.BooleanQuery ();
@@ -586,10 +597,51 @@ namespace Beagle.Daemon {
 		// Code to map to/from Lucene data types
 		//
 
+		private void AddPropertyToLuceneDocument (Document doc, Property prop)
+		{
+			if (prop.Value == null)
+				return;
+
+			Field f;
+
+			if (prop.IsSearched) {
+				f = new Field (prop.IsKeyword ? "PropertyKeyword" : "PropertyText",
+					       prop.Value,
+					       false,              // never stored
+					       true,               // always index
+					       ! prop.IsKeyword);  // only tokenize non-keywords
+				doc.Add (f);
+			}
+
+			f = new Field (String.Format ("prop:{0}:{1}",
+						      prop.IsKeyword  ? "k" : "_",
+						      prop.Key),
+				       prop.Value,
+				       true,               // always store
+				       true,               // always index
+				       ! prop.IsKeyword);  // only tokenize non-keywords
+			doc.Add (f);
+		}
+
+		private Property FieldToProperty (Field field)
+		{
+			string name = field.Name ();
+			if (name.Length < 7 || ! name.StartsWith ("prop:"))
+				return null;
+
+			string key = name.Substring (7);
+			string value = field.StringValue ();
+			
+			if (name [5] == 'k')
+				return Property.NewKeyword (key, value);
+			else if (name [6] == 's')
+				return Property.NewUnsearched (key, value);
+			else
+				return Property.New (key, value);
+		}
+
 		private Document ToLuceneDocument (Indexable indexable)
 		{			
-			indexable.Build ();
-			
 			Document doc = new Document ();
 			Field f;
 			String str;
@@ -598,7 +650,7 @@ namespace Beagle.Daemon {
 			// First we add the Indexable's 'canonical' properties
 			// to the Document.
 			
-			f = Field.Keyword ("Uri", indexable.Uri.ToString ());
+			f = Field.Keyword ("Uri", UriFu.UriToSerializableString (indexable.Uri));
 			doc.Add (f);
 
 			f = Field.Keyword ("Type", indexable.Type);
@@ -633,161 +685,203 @@ namespace Beagle.Daemon {
 				doc.Add (f);
 			}
 
-			f = Field.UnStored ("PropertiesText",
-					    indexable.TextPropertiesAsString);
-			doc.Add (f);
-
-			// FIXME: We shouldn't apply stemming, etc. when dealing
-			// with this field.
-			f = Field.UnStored ("PropertiesKeyword",
-					    indexable.KeywordPropertiesAsString);
-			doc.Add (f);
-			
-			// FIXME: We need to deal with duplicate properties in some
-			// sort of sane way.
-			foreach (Property prop in indexable.Properties) {
-				if (prop.Value != null) {
-					f = Field.Keyword (ToLucenePropertyKey (prop.Key),
-							   prop.Value);
-					doc.Add (f);
-				}
-			}
+			foreach (Property prop in indexable.Properties)
+				AddPropertyToLuceneDocument (doc, prop);
 			
 			return doc;
 		}
-		
-		private LNS.Query ToCoreLuceneQuery (QueryBody body, string field)
+
+		static public LNS.Query ToUriQuery (ICollection list_of_uris, UriRemapper remapper)
 		{
-			LNS.BooleanQuery luceneQuery = null;
-			foreach (string text_orig in body.Text) {
-				string text = text_orig;
+			if (list_of_uris == null || list_of_uris.Count == 0)
+				return null;
 
-				if (text == null || text == "")
-					continue;
+			LNS.BooleanQuery query = new LNS.BooleanQuery ();
+			int max_clauses = LNS.BooleanQuery.GetMaxClauseCount ();
+			int clause_count = 0;
 
-				bool minus_sign = false;
-				if (text [0] == '-') {
-					text = text.Substring (1);
-					minus_sign = true;
-				}
-
-				// Use the analyzer to extract the query's tokens.
-				// This code is taken from Lucene's query parser.
-				// We use the standard Analyzer.
-				TokenStream source = LuceneDriver.Analyzer.TokenStream (field, new StringReader (text));
-				ArrayList tokens = new ArrayList ();
-
-				while (true) {
-					Lucene.Net.Analysis.Token t;
-					try {
-						t = source.Next ();
-					} catch (IOException) {
-						t = null;
-					}
-					if (t == null)
-						break;
-					tokens.Add (t.TermText ());
-				}
-				try {
-					source.Close ();
-				} catch (IOException) { 
-					// ignore
-				}
-
-				LNS.Query q = null;
-				if (tokens.Count == 1) {
-					Term t = new Term (field, (string) tokens [0]);
-					q = new LNS.TermQuery (t);
-				} else if (tokens.Count > 1) {
-					q = new LNS.PhraseQuery ();
-					foreach (string tokenStr in tokens) {
-						Term t = new Term (field, tokenStr);
-						((LNS.PhraseQuery) q).Add (t);
-					}
-				}
-
-				if (q != null) {
-					if (luceneQuery == null)
-						luceneQuery = new LNS.BooleanQuery ();
-					luceneQuery.Add (q, !minus_sign, minus_sign);
+			foreach (Uri original_uri in list_of_uris) {
+				Uri uri = original_uri;
+				if (remapper != null)
+					uri = remapper (uri);
+				Term term = new Term ("Uri", uri.ToString ()); // FIXME: Do we need some UriFu here?
+				LNS.Query term_query = new LNS.TermQuery (term);
+				query.Add (term_query, false, false);
+				++clause_count;
+				// If we have to many clases, nest the queries
+				if (clause_count == max_clauses) {
+					LNS.BooleanQuery new_query = new LNS.BooleanQuery ();
+					new_query.Add (query, false, false);
+					query = new_query;
+					clause_count = 1;
 				}
 			}
-			return luceneQuery;
 
+			return query;
 		}
 
-		private LNS.Query ToLuceneQuery (QueryBody body,
-						 ICollection listOfUris)
+		static public LNS.Query ToUriQuery (ICollection list_of_uris)
 		{
-			LNS.BooleanQuery luceneQuery = new LNS.BooleanQuery ();
+			return ToUriQuery (list_of_uris, null);
+		}
+
+		static private LNS.Query NewTokenizedQuery (string field, string text)
+		{
+			ArrayList tokens = new ArrayList ();
+
+			// Use the analyzer to extract the query's tokens.
+			// This code is taken from Lucene's query parser.
+			// We use the standard Analyzer.
+			TokenStream source = LuceneDriver.Analyzer.TokenStream (field, new StringReader (text));
+			while (true) {
+				Lucene.Net.Analysis.Token t;
+				try {
+					t = source.Next ();
+				} catch (IOException) {
+					t = null;
+				}
+				if (t == null)
+					break;
+				tokens.Add (t.TermText ());
+			}
+			try {
+				source.Close ();
+			} catch (IOException) { 
+				// ignore
+			}
 			
-			if (body.Text.Count > 0) {
-				LNS.BooleanQuery contentQuery = new LNS.BooleanQuery ();
-
-				LNS.Query propTQuery;
-				propTQuery = ToCoreLuceneQuery (body, "PropertiesText");
-				if (propTQuery != null) {
-					propTQuery.SetBoost (2.5f);
-					contentQuery.Add (propTQuery, false, false);
+			LNS.Query q = null;
+			if (tokens.Count == 1) {
+				Term t = new Term (field, (string) tokens [0]);
+				q = new LNS.TermQuery (t);
+			} else if (tokens.Count > 1) {
+				q = new LNS.PhraseQuery ();
+				foreach (string tokenStr in tokens) {
+					Term t = new Term (field, tokenStr);
+					((LNS.PhraseQuery) q).Add (t);
 				}
+			}
 
-				LNS.Query propKQuery;
-				propKQuery = ToCoreLuceneQuery (body, "PropertiesKeyword");
-				if (propKQuery != null) {
-					propKQuery.SetBoost (2.5f);
-					contentQuery.Add (propKQuery, false, false);
+			return q;
+		}
+
+		// search_subset limits the score of our search to that set of Uris
+		// bonus_uris are always matched by the query
+		private LNS.Query ToLuceneQuery (Query query,
+						 ICollection search_subset,
+						 ICollection bonus_uris)
+		{
+			LNS.BooleanQuery body_query = null;
+			LNS.Query        search_subset_query = null;
+			LNS.Query        bonus_uris_query = null;
+			LNS.BooleanQuery mime_type_query = null;
+			LNS.BooleanQuery hit_type_query = null;
+			
+			if (query.Parts.Count > 0) {
+
+				body_query = new LNS.BooleanQuery ();
+
+				foreach (QueryPart part in query.Parts) {
+
+					LNS.Query subquery = null;
+
+					if (part.TargetIsAll || part.TargetIsText) {
+
+						subquery = NewTokenizedQuery ("Text", part.Text);
+						body_query.Add (subquery, part.IsRequired, part.IsProhibited);
+
+						subquery = NewTokenizedQuery ("HotText", part.Text);
+						subquery.SetBoost (1.75f);
+						body_query.Add (subquery, part.IsRequired, part.IsProhibited);
+					}
+
+					if (part.TargetIsAll || part.TargetIsProperties) {
+						subquery = NewTokenizedQuery ("PropertyText", part.Text);
+						body_query.Add (subquery, part.IsRequired, part.IsProhibited);
+					}
+
+					if (part.TargetIsSpecificProperty) {
+
+						string prop_name;
+						prop_name = String.Format ("prop:{0}:{1}",
+									   part.IsKeyword ? 'k' : '_',
+									   part.Target);
+						
+						if (part.IsKeyword) {
+							Term term = new Term (prop_name, part.Text);
+							subquery = new LNS.TermQuery (term);
+						} else {
+							subquery = NewTokenizedQuery (prop_name, part.Text);
+						}
+
+						body_query.Add (subquery, part.IsRequired, part.IsProhibited);
+					}
 				}
+			}
+
+			search_subset_query = ToUriQuery (search_subset, null);
+
+			bonus_uris_query = ToUriQuery (bonus_uris, null);
 				
-				LNS.Query hotQuery;
-				hotQuery = ToCoreLuceneQuery (body, "HotText");
-				if (hotQuery != null) {
-					hotQuery.SetBoost (1.75f);
-					contentQuery.Add (hotQuery, false, false);		
-				}
-				
-				LNS.Query textQuery;
-				textQuery = ToCoreLuceneQuery (body, "Text");
-				if (textQuery != null) {
-					contentQuery.Add (textQuery, false, false);
-				}
-
-				luceneQuery.Add (contentQuery, true, false);
-			}
-
-			// If mime types are specified, we must match one of them.
-			if (body.MimeTypes.Count > 0) {
-				LNS.BooleanQuery mimeTypeQuery = new LNS.BooleanQuery ();
-				foreach (string mimeType in body.MimeTypes) {
-					Term t = new Term ("MimeType", mimeType);
+			if (query.MimeTypes.Count > 0) {
+				mime_type_query = new LNS.BooleanQuery ();
+				foreach (string mime_type in query.MimeTypes) {
+					Term t = new Term ("MimeType", mime_type);
 					LNS.Query q = new LNS.TermQuery (t);
-					mimeTypeQuery.Add (q, false, false);
+					mime_type_query.Add (q, false, false);
 				}
-				luceneQuery.Add (mimeTypeQuery, true, false);
 			}
 
-			// If hit types are specified, we must match one of them.
-			if (body.HasHitTypes) {
-				LNS.BooleanQuery hitTypeQuery = new LNS.BooleanQuery ();
-				foreach (string hitType in body.HitTypes) {
-					Term t = new Term ("Type", hitType);
+			if (query.HasHitTypes) {
+				hit_type_query = new LNS.BooleanQuery ();
+				foreach (string hit_type in query.HitTypes) {
+					Term t = new Term ("Type", hit_type);
 					LNS.Query q = new LNS.TermQuery (t);
-					hitTypeQuery.Add (q, false, false);
+					hit_type_query.Add (q, false, false);
 				}
-				luceneQuery.Add (hitTypeQuery, true, false);
 			}
 
-			// If a list of Uris is specified, we must match one of them.
-			if (listOfUris != null && listOfUris.Count > 0) {
-				LNS.BooleanQuery uriQuery = new LNS.BooleanQuery ();
-				foreach (Uri uri in listOfUris) {
-					Term t = new Term ("Uri", uri.ToString ());
-					LNS.Query q = new LNS.TermQuery (t);
-					uriQuery.Add (q, false, false);
-				}
-				luceneQuery.Add (uriQuery, true, false);
+			//
+			// Now we combine the various parts into one big query.
+			//
+
+			LNS.BooleanQuery total_query = new LNS.BooleanQuery ();
+
+			// If we have hit types or mime types, those must be matched
+			if (mime_type_query != null)
+				total_query.Add (mime_type_query, true, false);
+			if (hit_type_query != null)
+				total_query.Add (hit_type_query, true, false);
+
+			// We also must match the "content query":
+			// (body_query OR bonus_uris_query) AND search_subset_query
+
+			LNS.Query content_query = null;
+
+			if (body_query != null && bonus_uris_query != null) {
+				LNS.BooleanQuery q = new LNS.BooleanQuery ();
+				q.Add (body_query, false, false);
+				q.Add (bonus_uris_query, false, false);
+				content_query = q;
+			} else if (body_query != null) {
+				content_query = body_query;
+			} else if (bonus_uris_query != null) {
+				content_query = bonus_uris_query;
 			}
 
-			return luceneQuery;
+			if (content_query != null && search_subset_query != null) {
+				LNS.BooleanQuery q = new LNS.BooleanQuery ();
+				q.Add (content_query, true, false);
+				q.Add (search_subset_query, true, false);
+				content_query = q;
+			} else if (search_subset_query != null) {
+				content_query = search_subset_query;
+			}
+
+			if (content_query != null)
+				total_query.Add (content_query, true, false);
+
+			return total_query;
 		}
 		
 		static private Uri UriFromLuceneDoc (Document doc)
@@ -795,7 +889,7 @@ namespace Beagle.Daemon {
 			string uri = doc.Get ("Uri");
 			if (uri == null)
 				throw new Exception ("Got document from Lucene w/o a URI!");
-			return new Uri (uri, true);
+			return UriFu.UriStringToUri (uri);
 		}
 
 		static private void FromLuceneDocToVersioned (Document doc, Versioned versioned)
@@ -835,9 +929,9 @@ namespace Beagle.Daemon {
 			hit.ScoreRaw = score;
 			
 			foreach (Field ff in doc.Fields ()) {
-				string key = FromLucenePropertyKey (ff.Name ());
-				if (key != null)
-					hit [key] = ff.StringValue ();
+				Property prop = FieldToProperty (ff);
+				if (prop != null)
+					hit.AddProperty (prop);
 			}
 			
 			return hit;
@@ -850,12 +944,124 @@ namespace Beagle.Daemon {
 		// A common, shared analyzer
 		//
 
+		private class BeagleNoiseFilter : TokenFilter {
+			
+			static int total_count = 0;
+			static int noise_count = 0;
+
+			TokenStream token_stream;
+
+			public BeagleNoiseFilter (TokenStream input) : base (input)
+			{
+				token_stream = input;
+			}
+
+			// FIXME: we should add some heuristics that are stricter
+			// but explicitly try to avoid filtering out dates,
+			// phone numbers, etc.
+			private static bool IsNoise (string text)
+			{
+				// Anything really long is almost certainly noise.
+				if (text.Length > 30) 
+					return true;
+
+				// Look at how often we switch between numbers and letters.
+				// Scoring:
+				// <letter> <digit>   1
+				// <digit> <letter>   1
+				// <x> <punct>+ <x>   1
+				// <x> <punct>+ <y>   2
+				const int transitions_cutoff = 4;
+				int last_type = -1, last_non_punct_type = -1, first_type = -1;
+				bool has_letter = false, has_digit = false, has_punctuation = false;
+				int transitions = 0;
+				for (int i = 0; i < text.Length && transitions < transitions_cutoff; ++i) {
+					char c = text [i];
+					int type = -1;
+					if (Char.IsLetter (c)) {
+						type = 1;
+						has_letter = true;
+					} else if (Char.IsDigit (c)) {
+						type = 2;
+						has_digit = true;
+					} else if (Char.IsPunctuation (c)) {
+						type = 3;
+						has_punctuation = true;
+					}
+					
+					if (type != -1) {
+						
+						if (type != last_type) {
+							if (last_type == 3) {
+								if (type != last_non_punct_type)
+									++transitions;
+							} else {
+								++transitions;
+							}
+						}
+
+						if (first_type == -1)
+							first_type = type;
+
+						last_type = type;
+						if (type != 3)
+							last_non_punct_type = type;
+					}
+				}
+
+				// If we make too many transitions, it must be noise.
+				if (transitions >= transitions_cutoff) 
+					return true;
+
+				// If we consist of nothing but digits and punctuation, treat it
+				// as noise if it is too long.
+				if (transitions == 1 && first_type != 1 && text.Length > 10)
+					return true;
+
+				// We are very suspicious of long things that make lots of
+				// transitions
+				if (transitions > 3 && text.Length > 10) 
+					return true;
+
+				// Beware of anything long that contains a little of everything.
+				if (has_letter && has_digit && has_punctuation && text.Length > 10)
+					return true;
+
+				//Logger.Log.Debug ("BeagleNoiseFilter accepted '{0}'", text);
+				return false;
+				
+			}
+
+			public override Lucene.Net.Analysis.Token Next ()
+			{
+				Lucene.Net.Analysis.Token token;
+				while ( (token = token_stream.Next ()) != null) {
+#if false
+					if (total_count > 0 && total_count % 5000 == 0)
+						Logger.Log.Debug ("BeagleNoiseFilter filtered {0} of {1} ({2:0.0}%)",
+								  noise_count, total_count, 100.0 * noise_count / total_count);
+#endif
+					++total_count;
+					if (IsNoise (token.TermText ())) {
+						++noise_count;
+						continue;
+					}
+					return token;
+				}
+				return null;
+			}
+		}
+
 		// This is just a standard analyzer combined with the Porter stemmer.
 		// FIXME: This assumes everything being indexed is in English!
 		private class BeagleAnalyzer : StandardAnalyzer {
 			public override TokenStream TokenStream (String fieldName, TextReader reader)
 			{
-				return new PorterStemFilter (base.TokenStream (fieldName, reader));
+				TokenStream outstream = base.TokenStream (fieldName, reader);
+				if (fieldName == "Text" || fieldName == "HotText")
+					outstream = new BeagleNoiseFilter (outstream);
+				outstream = new PorterStemFilter (outstream);
+				return outstream;
 			}
 		}
 
@@ -898,18 +1104,23 @@ namespace Beagle.Daemon {
 			return Convert.ToInt64 (str);
 		}
 
-		const string propPrefix = "prop:";
+		/////////////////////////////////////////////////////
 
-		private string ToLucenePropertyKey (string key)
-		{
-			return propPrefix + key;
-		}
+		// Expose some information for debugging and analytical purposes.
 
-		private string FromLucenePropertyKey (string key)
+		public void WriteIndexTermFrequencies (TextWriter writer)
 		{
-			if (key.StartsWith (propPrefix))
-				return key.Substring (propPrefix.Length);
-			return null;
+			IndexReader reader = IndexReader.Open (Store);
+			TermEnum term_enum = reader.Terms ();
+
+			Term term;
+			while (term_enum.Next ()) {
+				term = term_enum.Term ();
+				int freq = term_enum.DocFreq ();
+				writer.WriteLine ("{0} {1} {2}", term.Field (), freq, term.Text ());
+
+			}
+			reader.Close ();
 		}
 	}
 }
